//
//  mexCPI.cpp
//  ResearchCpp
//
//  Created by YoonMinji on 2017. 5. 2..
//  Copyright YoonMinji. All rights reserved.
//

#include <math.h>
#include "mex.h"
#include "matrix.h"

#include <vector>
#include "time.h"

#define EPSILON  0.000000001

// Sparse matrix is expressed using outEdge
struct outEdge
{
    outEdge()
    {
        weight = 0;
    }
    
    double weight;
    std::vector<int> out;
};

// Main function
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    // Decode MATLAB
    int n = mxGetM(prhs[0]);
    outEdge* A = new outEdge[n];

    const mxArray *array_ptr = prhs[0];

    double  *pr, *pi;
    mwIndex *ir, *jc;
    mwSize  col, total=0;
    mwIndex starting_row_index, stopping_row_index, current_row_index;
    mwSize  size;
    
    // Get the starting positions of all four data arrays.
    pr = mxGetPr(array_ptr);
    pi = mxGetPi(array_ptr);
    ir = mxGetIr(array_ptr);
    jc = mxGetJc(array_ptr);
    
    // Display the nonzero elements of the sparse array.
    size = mxGetN(array_ptr);
    for (col = 0; col < size; col++)  {
        starting_row_index = jc[col];
        stopping_row_index = jc[col+1];
        if (starting_row_index == stopping_row_index)
            continue;
        else {
            for (current_row_index = starting_row_index; current_row_index < stopping_row_index; current_row_index++)  {
                A[ir[current_row_index]].out.push_back(col);
                ++total;
            }
        }
    }

    int seed = mxGetPr(prhs[1])[0] - 1;
    double c = mxGetPr(prhs[2])[0];

    double* r = new double[n];
    double* nq = new double[n];
    double* nq_prev = new double[n];
    for(int i = 0; i < n; i++)
    {
        r[i] = nq[i] = nq_prev[i] = 0;
    }

    /////////// START ////////////
    clock_t start = clock();
    double delta_score = 100;
    
    for(int i = 0; i < n; i++)
    {
        if(A[i].out.size() > 0)
	    {
    	    A[i].weight = 1/double(A[i].out.size());
	    }
    }
   
    if(seed == -1)
    {
        for(int i = 0; i < n; i++)
        {
            nq_prev[i] = c/n;
            r[i] = c/n;
        }
    }
    else
    {
        nq_prev[seed] = c;
        r[seed] = c;
    }
    
    int out_iter = 0;

    double* new_nq;
    double* old_nq;
     
    while(delta_score > EPSILON)
    {     
        if(out_iter % 2 == 0)
        {
            new_nq = nq;
            old_nq = nq_prev;
        }
        else
        {
            new_nq = nq_prev;
            old_nq = nq;
        }
        
        for(int i = 0; i < n; i++)
        {
            new_nq[i] = 0;
        }
        
        for(int i = 0; i < n; i++)
        {
	        if(old_nq[i] == 0)
                continue;
            if(A[i].weight == 0)
                continue;

            double delta = (1-c)*A[i].weight*old_nq[i];
            
            std::vector<int>::iterator j = A[i].out.begin();
            for(; j != A[i].out.end(); j++) 
            {
                new_nq[*j] += delta;
            }
        }
            
        delta_score = 0;
        for(int i = 0; i < n; i++)
        {
            delta_score += new_nq[i];
            r[i] += new_nq[i]; 
        }
        ++out_iter;
    }

    double r_sum = 0;
    for(int i = 0; i < n; i++)
        r_sum += r[i];
    for(int i = 0; i < n; i++)
        r[i] /= r_sum;
    
    double elapsedTime = (double)(clock() - start) / CLOCKS_PER_SEC;
    
    plhs[0] = mxCreateDoubleMatrix(n+1, 1, mxREAL);
    double* r_target = mxGetPr(plhs[0]);
    for(int i = 0; i < n; i++)
    {
        r_target[i] = r[i];
    }
    r_target[n] = elapsedTime;

    delete [] new_nq;
    delete [] old_nq;
    delete [] r;
    delete [] A;
}
