//
//  mexOSP_delete.cpp
//  ResearchCpp
//
//  Created by YoonMinji on 2017. 5. 2..
//  Copyright YoonMinji. All rights reserved.
//

#include <math.h>
#include "mex.h"
#include "matrix.h"

#include <vector>
#include "time.h"

// Sparse matrix is expressed using outEdge
struct outEdge
{
    outEdge()
    {
        weight = 0;
    }
    
    double weight;
    std::vector<int> out;
};

// Main function
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    //////////////////// Decode Initial Matrix A //////////////////////////////
    int n = mxGetM(prhs[0]);
    outEdge* A = new outEdge[n];

    const mxArray *array_ptr = prhs[0];
    double  *pr, *pi;
    mwIndex *ir, *jc;
    mwSize  col, total=0;
    mwIndex starting_row_index, stopping_row_index, current_row_index;
    mwSize  size;
    
    // Get the starting positions of all four data arrays.
    pr = mxGetPr(array_ptr);
    pi = mxGetPi(array_ptr);
    ir = mxGetIr(array_ptr);
    jc = mxGetJc(array_ptr);
    
    // Display the nonzero elements of the sparse array.
    size = mxGetN(array_ptr);
    for (col = 0; col < size; col++)  {
        starting_row_index = jc[col];
        stopping_row_index = jc[col+1];
        if (starting_row_index == stopping_row_index)
            continue;
        else {
            for (current_row_index = starting_row_index; current_row_index < stopping_row_index; current_row_index++)  {
                A[ir[current_row_index]].out.push_back(col);
                ++total;
            }
        }
    }
   
    //////////////////// Initialize /////////////////////////////////////
    double* final_score = new double[n];
    double* r = new double[n];
    double* nq = new double[n];
    double* nq_prev = new double[n];
    for(int i = 0; i < n; i++)
    {
        final_score[i] = r[i] = nq[i] = nq_prev[i] = 0;
    }
    
    //////////////////// Decode Delta Graph //////////////////////
    int m = mxGetM(prhs[1]);
    int** delta = new int*[m];
    for(int i = 0; i < m; i++)
        delta[i] = new int[2];
    int t = 0;
    for(col = 0; col < mxGetN(prhs[1]); col++)
    {
        for(current_row_index = 0; current_row_index < mxGetM(prhs[1]); current_row_index++)
        {
            delta[current_row_index][col] = (mxGetPr(prhs[1]))[current_row_index + col*mxGetM(prhs[1])] - 1;
        }
    }

    /////////////////// Decode Initial RWR ////////////////////////////
    const mxArray *array_ptr2 = prhs[2];
    pr = mxGetPr(array_ptr2);
    pi = mxGetPi(array_ptr2);
    ir = mxGetIr(array_ptr2);
    jc = mxGetJc(array_ptr2);
    
    size = mxGetN(array_ptr2);
    total = 0;
    double init_length = 0;
    for (col = 0; col < size; col++)  {
        starting_row_index = jc[col];
        stopping_row_index = jc[col+1];
        if (starting_row_index == stopping_row_index)
            continue;
        else {
            for (current_row_index = starting_row_index; current_row_index < stopping_row_index; current_row_index++)  {
                final_score[ir[current_row_index]] = pr[total];
                init_length += pr[total];
                ++total;
            }
        }
    }
    
    //////////////////// PARAMETERS ////////////////////////////////   
    double c = mxGetPr(prhs[3])[0];
    double epsilon = mxGetPr(prhs[4])[0];
    int numDelete = mxGetPr(prhs[5])[0]; 


    //////////////////////////////
    /////////// START ////////////
    //////////////////////////////
    for(int i = 0; i < n; i++)
    {
        if(A[i].out.size() > 0)
	    {
    	    A[i].weight = 1/double(A[i].out.size());
	    }
    }
    
    clock_t start = clock();
    double* new_nq;
    double* old_nq;
    
    ////////////////// CALCULATE THE UPDATED ADJACENCY MATRIX //////////////////////////////
    for(int update = 1; update < numDelete + 1; update++)
    {
        int u = delta[m-update][0];
        int v = delta[m-update][1];

        int pre_size = A[u].out.size();
        
        if(pre_size == 1)
        {
            nq_prev[v] -= 1/double(pre_size)*final_score[u];
            r[v] -= 1/double(pre_size)*final_score[u];
        
            A[u].weight = 0;
        }
        else
        {
            for(std::vector<int>::iterator i = A[u].out.begin(); i != A[u].out.end(); )
            {
                nq_prev[*i] += (1/double(pre_size-1)-1/double(pre_size))*final_score[u];
                r[*i] += (1/double(pre_size-1)-1/double(pre_size))*final_score[u];
                
                if(*i == v)
                    i = A[u].out.erase(i);
                else
                    i++;
            }
            
            nq_prev[v] -= 1/double(pre_size-1)*final_score[u];
            r[v] -= 1/double(pre_size-1)*final_score[u];
            
            A[u].weight = 1/double(pre_size-1);
        }
    }

    double initial = 0; 
    for(int i = 0; i < n; i ++)
    {
        nq_prev[i] *= (1-c);
        initial += r[i] > 0 ? r[i] : -1*r[i];
        r[i] *= (1-c);
    }
    
    ////////////////////// PROPAGATE OFFSET SCORES ///////////////////////////////////////////
    double delta_score = 100;
    int out_iter = 0;
    long long int visited = 0;
    while(delta_score > epsilon)
    {     
        if(out_iter % 2 == 0)
        {
            new_nq = nq;
            old_nq = nq_prev;
        }
        else
        {
            new_nq = nq_prev;
            old_nq = nq;
        }
        
        for(int i = 0; i < n; i++)
        {
            new_nq[i] = 0;
        }
        
        for(int i = 0; i < n; i++)
        {
            if(old_nq[i] == 0)
                continue;
            if(A[i].weight == 0)
                continue;

            double delta = (1-c)*A[i].weight*old_nq[i];
            std::vector<int>::iterator j = A[i].out.begin();
            for(; j != A[i].out.end(); j++) 
            {
                new_nq[*j] += delta;
                visited ++;
            }
        }
            
        delta_score = 0;
        for(int i = 0; i < n; i++)
        {
            delta_score += (new_nq[i] >= 0)? new_nq[i] : (-1)*new_nq[i];
            r[i] += new_nq[i]; 
        }
        ++out_iter;
    }

    ///////////////////// MERGE WITH THE PREVIOUS RWR SCORES ///////////////////////////
    double final_sum = 0;
    for(int i = 0; i < n; i++)
    {
        final_score[i] += r[i];
        final_sum += final_score[i];
    }
    
    ///////////////////// HANDLINGE DEAD-ENDS /////////////////////////////////////////
    for(int i = 0; final_sum > 0 && i < n; i++)
    {
        final_score[i] /= final_sum;
    }

    double elapsedTime = (double)(clock() - start) / CLOCKS_PER_SEC;

    plhs[0] = mxCreateDoubleMatrix(n+1, 1, mxREAL);
    double* r_target = mxGetPr(plhs[0]);
    for(int i = 0; i < n; i++)
    {
        r_target[i] = final_score[i];
    }
    r_target[n] = elapsedTime;
    
    delete [] final_score;
    delete [] nq;
    delete [] nq_prev;
    delete [] r;
    delete [] A;

    for(int i = 0; i < m; i ++)
        delete [] delta[i];
    delete [] delta;
}
