mex mexCPI.cpp -largeArrayDims;
mex mexOSP_delete.cpp -largeArrayDims;

%%%%%%%%%%%%%%% PRARAMETERS %%%%%%%%%%%%%%%%%
[B, X, n] = Import('Mixed_Slashdot'); % random edge stream
seed = dlmread('Seedset_Slashdot', '\t');   % random seed set 
c = 0.15;                                   % restart probability
epsilon = 0.0001;                           % error tolerance for OSP-T
deleted = 10;                              % number of edges to be deleted

sd = zeros(1,1); sd(1,1) = 0;
cc = zeros(1,1); cc(1,1)= c;
ep = zeros(1,1); ep(1,1) = 0.0000000001;
de = zeros(1,1); de(1,1) = deleted;

fprintf('\n[OSP: dyanmic RWR computation]\n');

for i = 1:1 %length(seed)
    
    % ORIGINAL RWR
    sd(1,1) = seed(i);
    [R_C] = mexCPI(B, sd, cc);
    originRWR = R_C(1:n);
    fprintf('Original RWR computation time   : %g seconds\n', R_C(n+1));
   
    % OSP
    [R_C] = mexOSP_delete(B, X, sparse(originRWR), cc, ep, de); % B: original adjacency matrix, X: edge stream to be deleted 
    ospRWR = R_C(1:n);
    fprintf('OSP computation time            : %g seconds\n', R_C(n+1));
    
    % OSP-T
    ep(1,1) = epsilon;
    [R_C] = mexOSP_delete(B, X, sparse(originRWR), cc, ep, de); % B: original adjacency matrix, X: edge stream to be deleted 
    osp_t_RWR = R_C(1:n);
    fprintf('OSP-T computation time          : %g seconds\n', R_C(n+1));
    fprintf('OSP-T L1 error                  : %g\n', norm(ospRWR - osp_t_RWR, 1));
    fprintf('OSP-T spearman correlation      : %g\n\n', corr(ospRWR, osp_t_RWR, 'type', 'Spearman'));

end

